/**
 * Copyright (C) 2007  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.novabpm.identity.auth;

import java.lang.management.ManagementFactory;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.management.InstanceAlreadyExistsException;
import javax.management.MBeanRegistrationException;
import javax.management.MBeanServer;
import javax.management.MalformedObjectNameException;
import javax.management.NotCompliantMBeanException;
import javax.management.ObjectName;

import org.jbpm.pvm.env.Environment;
import org.jbpm.pvm.env.EnvironmentFactory;
import org.jbpm.pvm.env.PvmEnvironmentFactory;
import org.jbpm.pvm.env.Transaction;
import org.ow2.novabpm.util.Misc;

/**
 * @author "Pierre Vigneras"
 * @date Dec 14, 2007
 */
public class AuthenticationService implements AuthenticationServiceMBean {
	public static final String CONFIGURATION = "conf/authServiceConfig.xml";
	private final AuthenticationServiceOp service;

  private static final Logger logger = Logger
      .getLogger(AuthenticationService.class.getName());



  public AuthenticationService() {
	  final EnvironmentFactory factory = new PvmEnvironmentFactory(CONFIGURATION);
		Misc.badStateIfNull(factory, "Can't parse the environment from resource: " + CONFIGURATION);
    Environment environment = null;
    try {
      environment = factory.openEnvironment();
      this.service = environment.get(AuthenticationServiceOp.class);
      if (service == null)
        throw new IllegalArgumentException(
            "Can't get the AuthenticationServiceOp implementation to use from environment: "
                + environment);
      logger.config("Service: " + service + " taken from environment: "
          + environment);
    } catch (Exception e) {
      if (environment != null) {
        Transaction t = environment.get(Transaction.class);
        if (t != null) {
          t.setRollbackOnly();
        }
      }
      if (e instanceof RuntimeException) {
        throw (RuntimeException) e;
      } else {
        throw new RuntimeException(e);
      }
    } finally {
      if (environment != null) {
        environment.close();
      }
    }
  }

  /*
   * (non-Javadoc)
   * 
   * @see org.ow2.novabpm.identity.auth.AuthenticationServiceMBean#authenticateUser(java.lang.String[])
   */
  public String authenticateUser(String... params) {
    return service.authenticateUser(params);
  }

  public static void newServer() throws MBeanRegistrationException {
    final MBeanServer mbs = ManagementFactory.getPlatformMBeanServer();
    final String bindingName = "org.ow2.bonita.mbean:type=AuthenticationService";
    ObjectName name;
    try {
      name = new ObjectName(bindingName);
    } catch (MalformedObjectNameException mone) {
      final Error e = new Error("Ouch! How can it be that my object name: "
          + bindingName + " is malformed?");
      e.initCause(mone);
      throw e;
    }

    final AuthenticationService mbean = new AuthenticationService();

    try {
      mbs.registerMBean(mbean, name);
    } catch (NotCompliantMBeanException ncme) {
      final Error e = new Error("Ouch! How can it be that my mbean: " + mbean
          + " is not compliant?!");
      e.initCause(ncme);
    } catch (InstanceAlreadyExistsException e) {
      logger.log(Level.WARNING,
          "An AuthenticationService MBean has already been registered with the name "
              + name + " in the MBean server: " + mbs + ". Ignoring.");
    }

    try {
      logger.info("Waiting for clients");
      Thread.sleep(Long.MAX_VALUE);
    } catch (InterruptedException e) {
      logger.log(Level.WARNING, "Interrupted! Exiting.", e);
    }
  }

  public static void usage() {
    System.err.println("Usage: " + AuthenticationService.class.getName()
        + " [-test]");
    System.exit(1);
  }

  public static void main(String[] args) throws MBeanRegistrationException {
    if (args.length == 0) {
      AuthenticationService.newServer();
    } else if (args[0].equals("-test")) {
      AuthenticationService.testServer();
    } else
      usage();
  }

  private static void testServer() {
    // TODO Auto-generated method stub

  }

  public String getImplName() {
    return service.getClass().getName();
  }
}
